#include "options.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include "defines.h"
#include "statusbar.h"

#define NUM_OPTS 33


int RSIZE1,GSIZE,KPRET;
int RSIZE2,RSIZE3;
char GRID_COLOR[100];
char EXIT_COLOR[100];
char RECT_COLOR[100];
char TEXT_COLOR[100];
char ACTIVE_TEXT_COLOR[100];
char ACTIVE_RECT_COLOR[100];
char CANVAS_COLOR[100];
char APP_FONT[100];
char SOURCE_FONT[100];
int DO_ARROWS;
double ZOOM_STEP;
double ZOOM_MIN;
double ZOOM_MAX;
char ROOM_PREFIX[100];
char ROOM_POSTFIX[100];
char AREA_NAME[100];
char FILE_EXT[100];
char COMMENT[100];
char CREATOR_NAME[100];
char MUDLIB_TEMPLATE[100];
char SOUTH_IS[30];
char WEST_IS[30];
char EAST_IS[30];
char NORTH_IS[30];
char NORTHWEST_IS[30];
char NORTHEAST_IS[30];
char SOUTHEAST_IS[30];
char SOUTHWEST_IS[30];
char DOWN_IS[30];
char UP_IS[30];

struct _opt {
	char type;
	void *data_ptr;
};

char *opt[]={
"RSIZE1", "GSIZE", "KPRET", "GRID_COLOR", "EXIT_COLOR",
"RECT_COLOR","TEXT_COLOR","ACTIVE_TEXT_COLOR",
"ACTIVE_RECT_COLOR", "CANVAS_COLOR", "APP_FONT",
"SOURCE_FONT","DO_ARROWS","ZOOM_STEP","ZOOM_MIN",
"ZOOM_MAX", "ROOM_PREFIX","AREA_NAME","FILE_EXT",
"COMMENT","CREATOR_NAME","MUDLIB_TEMPLATE",
"SOUTH_IS",
"NORTH_IS",
"WEST_IS",
"EAST_IS",
"SOUTHWEST_IS",
"SOUTHEAST_IS",
"NORTHWEST_IS",
"NORTHEAST_IS",
"DOWN_IS",
"UP_IS",
"ROOM_POSTFIX"};

char opt_type[]="ddbsssssssssbfffsssssssssssssssss";

struct _opt aopt[NUM_OPTS];


int parse_config_file(char *fname)
{

	int uid;
	FILE* fl;
	int ch;
	char line[100];
	int lptr;
	

	if (access(fname,F_OK)==-1) return -1;

	fl=fopen(fname,"r");
	
	if (fl<=0)
	{
		mdialog_msg("Unable to open gereacreator.conf\nWill use default settings");
		return;
	}

	lptr=0;
	memset(line,0x0,100);

	do
	{
		ch=(int)getc(fl);
		if (ch!=EOF) {

			if (ch!='\n')
			{
				line[lptr]=ch;
				lptr++;
			} else {
				line[lptr]='\0';

				parse_line(line);
				lptr=0;
			}
		}
	}
	while (ch!=EOF);

	
	fclose(fl);
}


int parse_line(char *ln)
{
	char *value;
	char *constant;
	int i;
	int opt_isin;
	
	while (*(ln)==' ' || *(ln)=='\t') {ln++; }
	if (strlen(ln)==0) return;
	if (*(ln)=='#') return;
	
	constant=strtok(ln,"=");
	value=strtok(0,"=");
	if (value==0) {
	value=malloc(sizeof(char)*2);
	value[0]=' ';
	value[1]='\0';
	}
	
	opt_isin=-1;
	for (i=0; i<NUM_OPTS;i++)
	{
		if (strcmp(constant,opt[i])==0) opt_isin=i;
	}

	if (opt_isin==-1) {
	printf("Option %s doesnt exit, ignoring\n",constant);
	return -1;
	}
	
	int k;
	switch (opt_isin)
	{
		case 0: RSIZE1=atoi(value); RSIZE2=RSIZE1*2; RSIZE3=RSIZE1*4;
			break;
		case 1: GSIZE=atoi(value); break;
		case 2: KPRET=atoi(value); break;
		case 3: strcpy(GRID_COLOR,value); break;
		case 4: strcpy(EXIT_COLOR,value); break;
		case 5: strcpy(RECT_COLOR,value); break;
		case 6: strcpy(TEXT_COLOR,value); break;
		case 7: strcpy(ACTIVE_TEXT_COLOR,value); break;
		case 8: strcpy(ACTIVE_RECT_COLOR,value); break;
		case 9: strcpy(CANVAS_COLOR,value); break;
		case 10: strcpy(APP_FONT,value); break;
		case 11: strcpy(SOURCE_FONT,value); break;
		case 12: DO_ARROWS=atoi(value); break;
		case 13: ZOOM_STEP=atof(value); break;
		case 14: ZOOM_MIN=atof(value); break;
		case 15: ZOOM_MAX=atof(value); break;
		case 16: strcpy(ROOM_PREFIX,value); break;
		case 17: strcpy(AREA_NAME,value); break;
		case 18: strcpy(FILE_EXT,value); break;
		case 19: strcpy(COMMENT,value); break;
		case 20: strcpy(CREATOR_NAME,value); break;
		case 21: strcpy(MUDLIB_TEMPLATE,value); break;
		case 22: strcpy(SOUTH_IS,value); break;
		case 23: strcpy(NORTH_IS,value); break;
		case 24: strcpy(WEST_IS,value); break;
		case 25: strcpy(EAST_IS,value); break;
		case 26: strcpy(SOUTHWEST_IS,value); break;
		case 27: strcpy(SOUTHEAST_IS,value); break;
		case 28: strcpy(NORTHWEST_IS,value); break;
		case 29: strcpy(NORTHEAST_IS,value); break;
		case 30: strcpy(DOWN_IS,value); break;
		case 31: strcpy(UP_IS,value); break;
		case 32: strcpy(ROOM_POSTFIX,value); break;

		default: return -1; break;
			
	}

}

int load_options(char *fname)
{
	g_print("Loading options from %s\n",fname);
	parse_config_file(fname);
}

int default_options()
{
	RSIZE1=D_RSIZE1;
	RSIZE2=RSIZE1*2;
	RSIZE3=RSIZE1*4;
	GSIZE=D_GSIZE;
	KPRET=(int)D_KPRET;
	strcpy(GRID_COLOR,D_GRID_COLOR);
	strcpy(EXIT_COLOR,D_EXIT_COLOR);
	strcpy(RECT_COLOR,D_RECT_COLOR);
	strcpy(TEXT_COLOR,D_TEXT_COLOR);
	strcpy(ACTIVE_TEXT_COLOR,D_ACTIVE_TEXT_COLOR);
	strcpy(ACTIVE_RECT_COLOR,D_ACTIVE_RECT_COLOR);
	strcpy(CANVAS_COLOR,D_CANVAS_COLOR);
	strcpy(APP_FONT,D_APP_FONT);
	strcpy(SOURCE_FONT,D_SOURCE_FONT);
	DO_ARROWS=D_DO_ARROWS;
	ZOOM_STEP=D_ZOOM_STEP;
	ZOOM_MIN=D_ZOOM_MIN;
	ZOOM_MAX=D_ZOOM_MAX;
	strcpy(ROOM_PREFIX,D_ROOM_PREFIX);
	strcpy(AREA_NAME,D_AREA_NAME);
	strcpy(FILE_EXT,D_FILE_EXT);
	strcpy(COMMENT,D_COMMENT);
	strcpy(CREATOR_NAME,D_CREATOR_NAME);
	strcpy(MUDLIB_TEMPLATE,D_MUDLIB_TEMPLATE);
	strcpy(SOUTH_IS,D_SOUTH_IS);
	strcpy(WEST_IS,D_WEST_IS);
	strcpy(NORTH_IS,D_NORTH_IS);
	strcpy(EAST_IS,D_EAST_IS);
	strcpy(NORTHEAST_IS,D_NORTHEAST_IS);
	strcpy(NORTHWEST_IS,D_NORTHWEST_IS);
	strcpy(SOUTHWEST_IS,D_SOUTHWEST_IS);
	strcpy(SOUTHEAST_IS,D_SOUTHEAST_IS);
	strcpy(UP_IS,D_UP_IS);
	strcpy(DOWN_IS,D_DOWN_IS);
	strcpy(ROOM_PREFIX,D_ROOM_POSTFIX);
}

